@@ -4,11 +4,11 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: filelog.c,v 1.1 1995/11/11 23:08:58 chuck Exp root $" ;
+static char RCSid[] = "$Id: filelog.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <varargs.h>
+#include <stdarg.h>
 #include <fcntl.h>
 #include <time.h>
 #ifndef NO_SYSLOG
@@ -45,7 +45,6 @@ PRIVATE int filelog_init( xp, ap )
 	va_list	ap ;
 {
 	int				fd ;
-	struct stat		st ;
 	struct filelog *flp ;
 	char				*filename	= va_arg( ap, char * ) ;
 	int				flags			= va_arg( ap, int ) ;
@@ -54,19 +53,6 @@ PRIVATE int filelog_init( xp, ap )
 	if ( flp == NULL )
 		return( XLOG_ENOMEM ) ;
 
-	/*
-	 * For safety, don't open a logfile if it's a symlink - cdm
-	 */
-	if ( lstat ( filename, &st ) != 0 )
-		return( XLOG_EOPEN ) ;
-	if ( ( st.st_mode & S_IFLNK ) == S_IFLNK )
-	{
-#ifndef NO_SYSLOG
-		syslog(LOG_WARNING, "Log file %s is a symlink - open abort", filename);
-#endif /* NO_SYSLOG */
-		return( XLOG_EOPEN ) ;
-	}
-
 	if ( flags & O_CREAT )
 		fd = open( filename, flags, va_arg( ap, int ) ) ;
 	else
@@ -193,7 +179,6 @@ PRIVATE int filelog_write( xp, buf, len, flags, ap )
 	int 				action_flags	= ( xp->xl_flags | flags ) ;
 	int				msglen			= 0 ;
 	char				*percent_m_pos ;
-	struct stat		st ;
 	int				cc ;
 	int				status ;
 	time_t 			current_time ;
@@ -203,24 +188,6 @@ PRIVATE int filelog_write( xp, buf, len, flags, ap )
 	if ( flp->fl_state != FL_OPEN )
 		return( flp->fl_error ) ;
 
-	/*
-	 * For safety, don't write a logfile if it's a symlink - cdm
-	 * BOGUS - this should really be done by a separate daemon
-	 */
-
-#if 0
-	if ( fstat ( flp->fl_fd, &st ) != 0 )
-	{
-		flp->fl_state = FL_ERROR ;
-		flp->fl_error = XLOG_EFSTAT ;
-	}
-	if ( ( st.st_mode & S_IFLNK ) == S_IFLNK )
-	{
-		flp->fl_state = FL_ERROR ;
-		flp->fl_error = XLOG_EFSTAT ;
-	}
-#endif
-
 	(void) time( &current_time ) ;
 	tmp = localtime( &current_time ) ;
 	cc = Sprint( flp->fl_fd, "%d/%d/%d@%02d:%02d:%02d",
