@@ -4,13 +4,21 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: access.c,v 6.6 1993/06/15 23:25:57 panos Exp $" ;
+static char RCSid[] = "$Id: access.c,v 1.3 1999/10/13 02:51:03 bbraun Exp $" ;
 
 #include <syslog.h>
 #include <time.h>
 
+#ifdef LIBWRAP
+#include <tcpd.h>
+#include <signal.h>
+int deny_severity = LOG_INFO;
+int allow_severity = LOG_INFO;
+#endif
+
 #include "connection.h"
 #include "service.h"
+#include "server.h"
 #include "state.h"
 #include "addr.h"
 #include "access.h"
@@ -20,13 +28,19 @@ time_t time() ;
 
 void msg() ;
 
+#ifdef HAVE_LOADAVG
+extern double xgetloadavg() ;
+#endif
+
 struct name_value access_code_names[] =
 	{
 		{ "address",				(int) AC_ADDRESS			},
 		{ "time",					(int) AC_TIME				},
 		{ "fork",					(int) AC_FORK				},
 		{ "service_limit",		(int) AC_SERVICE_LIMIT	},
+		{ "per_source_limit",	(int) AC_PER_SOURCE_LIMIT	},
 		{ "process_limit",		(int) AC_PROCESS_LIMIT	},
+		{ "libwrap",            (int) AC_LIBWRAP        },
 		{ CHAR_NULL,				1								},
 		{ "UNKNOWN",				0								}
 	} ;
@@ -106,7 +120,30 @@ access_e access_control( sp, cp, check_mask )
 	register mask_t				*check_mask ;
 {
 	register struct service_config	*scp = SVC_CONF( sp ) ;
-	struct sockaddr_in					*sinp ;
+	struct sockaddr_in					*sinp = SOCKADDRIN_NULL;
+	int u, n;
+
+#ifdef LIBWRAP
+   struct request_info req;
+
+   request_init(&req, RQ_DAEMON, scp->sc_name, RQ_FILE, cp->co_descriptor, 0);
+   fromhost(&req);
+   if (!hosts_access(&req))
+   {
+		syslog(deny_severity, "refused connect from %s", conn_addrstr(cp));
+      return(AC_LIBWRAP);
+   }
+#endif
+
+#ifdef HAVE_LOADAVG
+	if ( scp->sc_max_load != 0 ) {
+		if ( xgetloadavg() >= scp->sc_max_load ) {
+			syslog(LOG_ERR, "refused connect from %s due to excessive load\n",
+				conn_addrstr(cp));
+			return( AC_LOAD );
+		}
+	}
+#endif
 
 	if ( CHECK( check_mask, CF_ADDRESS ) && 
 				( sinp = conn_address( cp ) ) != SOCKADDRIN_NULL &&
@@ -121,6 +158,32 @@ access_e access_control( sp, cp, check_mask )
 	if ( CHECK( check_mask, CF_SERVICE_LIMIT ) &&
 								SVC_RUNNING_SERVERS( sp ) >= SC_INSTANCES( scp ) )
 		return( AC_SERVICE_LIMIT ) ;
+
+	if( scp->sc_per_source != UNLIMITED )
+	{
+		if ( sinp == SOCKADDRIN_NULL ) sinp = conn_address ( cp ) ;
+		if ( sinp != SOCKADDRIN_NULL ) {
+			n = 0 ;
+			for ( u = 0 ; u < pset_count( SERVERS( ps ) ) ; u++ )
+			{
+				struct server *serp ;
+				connection_s *cop ;
+				
+				serp = SERP( pset_pointer( SERVERS( ps ), u ) ) ;
+
+				if ( (SERVER_SERVICE( serp ) == sp) &&
+					( cop = SERVER_CONNECTION( serp ) ) )
+				{
+					if ( cop->co_remote_address.sin_addr.s_addr ==
+						sinp->sin_addr.s_addr)
+							n++;
+				}
+			}
+
+			if ( n >= scp->sc_per_source )
+				return( AC_PER_SOURCE_LIMIT ) ;
+		}
+	}
 	
 	if ( CHECK( check_mask, CF_PROCESS_LIMIT ) && ps.ros.process_limit )
 	{
