@@ -8,7 +8,7 @@
 #define ACCESS_H
 
 /*
- * $Id: access.h,v 1.1 1996/01/24 19:29:19 chuck Exp $
+ * $Id: access.h,v 1.2 1999/10/13 02:51:03 bbraun Exp $
  */
 
 #include "defs.h"
@@ -30,7 +30,11 @@ typedef enum
       AC_TIME,             /* we do not accept requests at this time       */
       AC_SERVICE_LIMIT,    /* server limit would be exceeded for this      */
                            /* service                                      */
-      AC_PROCESS_LIMIT     /* total process limit would be exceeded        */
+		AC_PER_SOURCE_LIMIT, /* server limit would be exceeded for this      */
+		                     /* service and source address                   */
+      AC_PROCESS_LIMIT,    /* total process limit would be exceeded        */
+      AC_LIBWRAP,
+      AC_LOAD
    } access_e ;
 
 extern struct name_value access_code_names[] ;
