@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: addr.c,v 1.1 1996/01/24 19:29:19 chuck Exp $" ;
+static char RCSid[] = "$Id: addr.c,v 1.2 1999/10/15 05:18:39 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -18,6 +18,10 @@ static char RCSid[] = "$Id: addr.c,v 1.1 1996/01/24 19:29:19 chuck Exp $" ;
 #include "defs.h"
 #include "addr.h"
 
+#if !defined(HAVE_SNPRINTF)
+extern int      snprintf(char *, size_t, const char *, ...);
+#endif
+
 unsigned long inet_addr() ;
 char *malloc() ;
 int free() ;
@@ -157,7 +161,8 @@ PRIVATE status_e add( addr_list, cap )
  *
  * XXX: we need to work on the way two cap's are considered equal
  */
-PRIVATE status_e remove( addr_list, cap )
+
+PRIVATE status_e xremove( addr_list, cap )
 	pset_h addr_list ;
 	register struct comp_addr *cap ;
 {
@@ -178,6 +183,67 @@ PRIVATE status_e remove( addr_list, cap )
 	return( OK ) ;
 }
 
+/*
+ * Function allows the use of 0.0.0.0/24 style address ranges for access cntl.
+ *   --bbraun 10/26/98
+ */
+PRIVATE result_e explicit_mask( char *str_addr, statfunc op, pset_h addr_list)
+{
+	struct comp_addr			ca ;
+	char newaddr[16];
+	int val;
+	int a, b, c, d, mask;
+
+	val = sscanf(str_addr, "%d.%d.%d.%d/%d", &a, &b, &c, &d, &mask);
+	if( val < 5 )
+	{
+		return( CANT_PARSE );
+	}
+
+#ifdef DEBUG
+	syslog(LOG_ERR, "explicit_mask: %d.%d.%d.%d/%d\n", a, b, c, d, mask);
+#endif
+
+	snprintf(newaddr, 16, "%d.%d.%d.%d", a, b, c, d);
+
+	ca.addr = inet_addr(&newaddr);
+
+	if(mask == 32)
+	{
+		ca.addr_type = HOST_ADDR;
+		ca.mask      = 0xFFFFFFFF;
+	}
+	else
+	{
+		unsigned long i, n;
+
+		ca.addr_type = NET_ADDR ;
+	
+		/* this is binary 100000000000000000000000000000000 */
+		/* i = 2147483648; */
+		/* A much more reliable method */
+		i = ( 1 << ((sizeof(i)*8)-1));
+		ca.mask = 0;
+
+		/* Go through and set each bit to 1 */
+		for( n=mask; n > 0 ; n--)
+		{
+			ca.mask |= i;
+			i /= 2;
+		}
+
+	}
+
+	ca.addr = ntohl(ca.addr);
+
+	ca.addr &= ca.mask;
+
+#ifdef DEBUG
+	syslog(LOG_ERR, "explicit_mask: %x %x\n", ca.addr, ca.mask);
+#endif
+	
+	return( ( (*op)( addr_list, &ca ) == OK ) ? PARSED : ERROR ) ;
+}
 
 /*
  * Try to parse 'str_addr' as a symbolic net name
@@ -437,6 +503,7 @@ PRIVATE status_e addrlist_op( addr_list, op, str_addr )
 	int i ;
 	static result_e (*addr_parser[])() =
 		{
+			explicit_mask,
 			numeric_addr,
 			factorized_addr,
 			net_addr,
@@ -472,7 +539,7 @@ status_e addrlist_remove( addr_list, str_addr )
 	pset_h addr_list ;
 	char *str_addr ;
 {
-	return( addrlist_op( addr_list, remove, str_addr ) ) ;
+	return( addrlist_op( addr_list, xremove, str_addr ) ) ;
 }
 
 
