@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: builtins.c,v 1.5 1995/09/10 18:41:13 chuck Exp $" ;
+static char RCSid[] = "$Id: builtins.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -42,6 +42,7 @@ PRIVATE void stream_chargen() ;
 PRIVATE void dgram_chargen() ;
 PRIVATE void stream_servers() ;
 PRIVATE void stream_services() ;
+PRIVATE void xadmin_handler() ;
 
 
 static struct builtin_service builtin_services[] =
@@ -58,6 +59,7 @@ static struct builtin_service builtin_services[] =
 		{ "chargen",	SOCK_DGRAM,		{ dgram_chargen,		NO_FORK	} },
 		{ "servers",	SOCK_STREAM,	{ stream_servers,		FORK		} },
 		{ "services",	SOCK_STREAM,	{ stream_services,	FORK		} },
+		{ "xadmin",    SOCK_STREAM,   { xadmin_handler,    FORK     } },
 		{ NULL }
 	} ;
 
@@ -68,7 +70,7 @@ builtin_s *builtin_find( service_name, type )
 {
 	register builtin_s	*bsp ;
 	char						*func = "builtin_find" ;
-
+	
 	if ( bsp = builtin_lookup( builtin_services, service_name, type ) )
 		return( bsp ) ;
 	
@@ -83,7 +85,7 @@ builtin_s *builtin_lookup( services, service_name, type )
 	register int				type ;
 {
 	register struct builtin_service *bsp ;
-
+	
 	for ( bsp = services ; bsp->bs_name != NULL ; bsp++ )
 		if ( EQ( bsp->bs_name, service_name ) && bsp->bs_socket_type == type )
 			return( &bsp->bs_handle ) ;
@@ -181,25 +183,27 @@ PRIVATE void daytime_protocol( buf, buflen )
 	time_t		now ;
 	struct tm	*tmp ;
 	int			size = *buflen ;
+#ifdef HAVE_STRFTIME
+	int		   cc ;
+#endif
 	
-#ifdef linux
-	char tzname[2];
-
-	(void) time( &now ) ;
-	tmp = localtime( &now ) ;
-	mktime(tmp);
-        strx_print( buflen, buf, size,
-                "%02d %s %d %02d:%02d:%02d %c%cT\r\n",
-                tmp->tm_mday, month_name[ tmp->tm_mon ], 1900 + tmp->tm_year,
-                tmp->tm_hour, tmp->tm_min, tmp->tm_sec, tzname[0], tzname[1] ) ;
-#else
 	(void) time( &now ) ;
 	tmp = localtime( &now ) ;
+#ifndef HAVE_STRFTIME
 	strx_print( buflen, buf, size,
 		"%02d %s %d %02d:%02d:%02d %s\r\n",
-		tmp->tm_mday, month_name[ tmp->tm_mon ], 1900 + tmp->tm_year,
-		tmp->tm_hour, tmp->tm_min, tmp->tm_sec, tmp->tm_zone ) ;
-#endif	/* linux */
+			tmp->tm_mday, month_name[ tmp->tm_mon ], 1900 + tmp->tm_year,
+				tmp->tm_hour, tmp->tm_min, tmp->tm_sec, tmp->tm_zone ) ;
+#else
+	cc = strx_nprint( buf, size,
+		"%02d %s %d %02d:%02d:%02d",
+			tmp->tm_mday, month_name[ tmp->tm_mon ], 1900 + tmp->tm_year,
+				tmp->tm_hour, tmp->tm_min, tmp->tm_sec ) ;
+	*buflen = cc ;
+	size -= cc ;
+	cc = strftime( &buf[ *buflen ], size, " %Z\r\n", tmp ) ;
+	*buflen += cc ;
+#endif
 }
 
 
@@ -232,11 +236,8 @@ PRIVATE void dgram_daytime( serp )
 	(void) sendto( descriptor, time_buf, buflen, 0, SA(&sin), sizeof( sin ) ) ;
 }
 
-#if defined(linux) || defined(__FreeBSD__) || defined(__bsdi__) || defined(__NetBSD__)
-#define TIME_OFFSET			2208988800UL
-#else
-#define TIME_OFFSET			2208988800
-#endif
+
+#define TIME_OFFSET			220898880
 
 /*
  * We always report the time using network-byte-order
@@ -416,3 +417,132 @@ PRIVATE void stream_services( serp )
 	}
 }
 
+#define MAX_CMD 100
+
+/* Do the redirection of a service */
+/* This function gets called from child.c after we have been forked */
+PRIVATE void xadmin_handler( struct server *serp )
+{
+	struct service *sp = SERVER_SERVICE( serp );
+	struct service_config *scp = SVC_CONF( sp );
+	int descriptor = SERVER_FD( serp );
+	char cmd[MAX_CMD]; 
+	int red = 0, u = 0;
+	int srchflags = STRS_BF | STRS_NOMALLOC | STRS_IGNCASE | STRS_PATLEN;
+	
+	while(1)
+	{
+		Sprint(descriptor, "> ");
+		Sflush();
+		bzero(cmd, MAX_CMD);
+		red = read(descriptor, cmd, MAX_CMD);
+		if( red < 0 )
+		{
+			syslog(LOG_ERR, "xadmin:reading command: %s\n", strerror(errno));
+			exit(1);
+		}
+		if( red > MAX_CMD )
+		{
+			/* shouldn't ever happen */
+			syslog(LOG_ERR, "xadmin:reading command: read more bytes than MAX_CMD\n");
+			continue;
+		}
+
+		if( red == 0 )
+			exit(0);
+
+	  if( (strncmp(cmd, "bye",(red<3)?red:3) == 0) ||
+	    (strncmp(cmd, "exit", (red<4)?red:4) == 0) ||
+	    (strncmp(cmd, "quit", (red<4)?red:4) == 0) )
+	  {
+		  Sprint(descriptor, "bye bye\n");
+		  Sflush();
+		  close(descriptor);
+		  exit(0);
+	  }
+
+     if( strncmp(cmd, "help", (red<4)?red:4) == 0 ) 
+     {
+        Sprint(descriptor, "xinetd admin help:\n");
+        Sprint(descriptor, "show run  :   shows information about running services\n");
+        Sprint(descriptor, "show avail:   shows what services are currently available\n");
+        Sprint(descriptor, "bye, exit :   exits the admin shell\n");
+     }
+
+	  if( strncmp(cmd, "show", (red<4)?red:4) == 0 )
+	  {
+		  char *tmp = cmd+4;
+		  red -= 4;
+		  if( tmp[0] == ' ' )
+		  {
+		  	tmp++;
+			red--;
+		  }
+
+		  if( red <= 0 )
+		  	continue;
+
+		  if( strncmp(tmp, "run", (red<3)?red:3) == 0 )
+		  {
+			  Sprint(descriptor, "Running services:\n");
+           Sprint(descriptor, "service  run retry attempts descriptor\n");
+			  for( u = 0 ; u < pset_count( SERVERS( ps ) ) ; u++ )
+			  {
+				  struct service *sp = 
+					(SERP( pset_pointer( SERVERS(ps), u ) ))->svr_sp;
+               
+              server_dump( SERP( pset_pointer( SERVERS(ps), u ) ), descriptor );
+  
+#ifdef FOO
+				  Sprint(descriptor, "%-10s %-3d %-5d %-7d %-5d\n", 
+					(char *)SVC_ID( sp ), sp->svc_running_servers+1, 
+					sp->svc_retry_servers, sp->svc_attempts, sp->svc_fd );
+#endif
+			  }
+		  }
+		  else if( strncmp(tmp, "avail", (red<5)?red:5) == 0 )
+		  {
+		
+			  Sprint(descriptor, "Available services:\n");
+			  Sprint(descriptor, "service    port   bound address    uid redir addr redir port\n");
+
+			  for( u = 0 ; u < pset_count( SERVICES( ps ) ) ; u++ )
+			  {
+				  struct service *sp=NULL;
+				  unsigned long baddr=0;
+
+				  sp = SP( pset_pointer( SERVICES( ps ), u ) );
+
+				  if( SVC_CONF(sp)->sc_bind_addr != NULL )
+				    baddr = SVC_CONF(sp)->sc_bind_addr->s_addr;
+  
+				 if( SVC_CONF(sp)->sc_redir_addr != NULL )
+				 {
+				   Sprint(descriptor, "%-10s ", SC_NAME( SVC_CONF( sp ) ) );
+				   Sprint(descriptor, "%-6d ", SC_PORT( SVC_CONF( sp ) ) );
+				   Sprint(descriptor, "%-16s ", inet_ntoa( baddr ) );
+				   Sprint(descriptor, "%-6d ", SVC_CONF(sp)->sc_uid );
+				   Sprint(descriptor, "%-16s ", inet_ntoa(SVC_CONF(sp)->sc_redir_addr->s_addr) );
+				   Sprint(descriptor, "%-6d\n", SVC_CONF(sp)->sc_redir_port );
+				 }
+				 else
+				 {
+				   Sprint(descriptor, "%-10s ", SC_NAME( SVC_CONF( sp ) ) );
+				   Sprint(descriptor, "%-6d ", SC_PORT( SVC_CONF( sp ) ) );
+				   Sprint(descriptor, "%-16s ", inet_ntoa( baddr ) );
+				   Sprint(descriptor, "%-6d\n", SVC_CONF(sp)->sc_uid );
+				 }
+
+			  }
+		  }
+		  else
+		  {
+			  Sprint(descriptor, "Relevant commands:\n");
+			  Sprint(descriptor, "run   : Show currently running servers\n");
+			  Sprint(descriptor, "avail : Show currently available servers\n");
+		  }
+	  }
+  }
+	Sprint(descriptor, "exiting\n");
+	Sflush();
+}
