@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: child.c,v 6.6 1993/06/13 01:42:13 panos Exp $" ;
+static char RCSid[] = "$Id: child.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -14,6 +14,8 @@ static char RCSid[] = "$Id: child.c,v 6.6 1993/06/13 01:42:13 panos Exp $" ;
 #include <netinet/in.h>
 #include <syslog.h>
 #include <errno.h>
+#include <pwd.h>
+#include <fcntl.h>
 
 #include "str.h"
 #include "pset.h"
@@ -32,8 +34,11 @@ void msg() ;
 void msg_suspend() ;
 void msg_resume() ;
 
+extern void redir_handler(struct server *serp);
 
-
+/*
+ * This function is running in the new process
+ */
 PRIVATE void exec_server( serp )
 	struct server *serp ;
 {
@@ -44,16 +49,36 @@ PRIVATE void exec_server( serp )
 	char *func = "exec_server" ;
 	void no_control_tty() ;
 
+	/*
+	 * The following code solves a problem with post-version-4.3
+	 * Ultrix systems (the bug was reported, and a fix was provided by
+	 * doug@seas.smu.edu; a slightly modified version of this
+	 * fix is included here).
+	 *
+	 * If this is a 'nowait' service, we pass the service descriptor
+	 * to the server. Note that we have set the close-on-exec flag
+	 * on all service descriptors. It is unclear whether the dup2()
+	 * will create a descriptor with the close-on-exec flag set,
+	 * so we explicitly clear the flag (since we are doing this
+	 * after the fork, it does not affect the descriptor of the
+	 * parent process).
+	 */
+	if ( fcntl( descriptor, F_SETFD, 0 ) == -1 )
+		msg( LOG_WARNING, func,
+			"fcntl( %d, clear close-on-exec ) failed: %m", descriptor ) ;
+
 	if ( debug.on )
 		msg( LOG_DEBUG, func, "duping %d", descriptor ) ;
 
 	for ( fd = 0 ; fd <= MAX_PASS_FD ; fd++ )
+	{
 		if ( dup2( descriptor, fd ) == -1 )
 		{
 			msg( LOG_ERR, func,
 					"dup2( %d, %d ) failed: %m", descriptor, fd ) ;
 			_exit( 1 ) ;
 		}
+	}
 
 	(void) close( descriptor ) ;
 
@@ -132,6 +157,49 @@ PRIVATE void set_credentials( scp )
 				msg( LOG_ERR, func, "setgid failed: %m" ) ;
 				_exit( 1 ) ;
 			}
+
+#ifndef NO_INITGROUPS
+			/*
+			 * Bug discovered by maf+@osu.edu (a bug fix was also provided;
+			 * a slightly modified version is included here):
+			 *		initgroups was not being invoked to set the remaining
+			 *		groups appropriately
+			 */
+			/* Solar Designer's groups fix */
+			if ( SC_SPECIFIED( scp, A_USER ) && SC_SPECIFIED( scp, A_GROUPS ) &&
+				scp->sc_groups == YES )
+			{
+				struct passwd *pwd ;
+
+				/*
+				 * Invoke getpwuid() to get the user's name.
+				 *
+				 * XXX:	we should not need to invoke getpwuid(); we should
+				 *			remember the user name in the configuration file.
+				 */
+				if ( ( pwd = getpwuid( SC_UID( scp ) ) ) == NULL )
+				{
+					msg( LOG_ERR, func, "getpwuid( %d ) (service=%s) failed: %m",
+						SC_UID( scp ), SC_ID( scp ) ) ;
+					_exit( 1 ) ;
+				}
+
+				if ( initgroups( pwd->pw_name, pwd->pw_gid ) == -1 )
+				{
+					msg( LOG_ERR, func, "initgroups( %s, %d ) failed: %m",
+						pwd->pw_name, pwd->pw_gid ) ;
+					_exit( 1 ) ;
+				}
+			}
+			else
+			{
+				if ( setgroups( 0, NULL ) )
+				{
+					msg( LOG_ERR, func, "setgroups( 0, NULL ) failed: %m" ) ;
+					_exit( 1 ) ;
+				}
+			}
+#endif	/* ! NO_INITGROUPS */
 		}
 		else
 			msg( LOG_WARNING, func, "can't change gid; not superuser" ) ;
@@ -161,6 +229,7 @@ void child_process( serp )
 	register struct server *serp ;
 {
 	struct service							*sp = SERVER_SERVICE( serp ) ;
+	connection_s                     *cp = SERVER_CONNECTION( serp ) ;
 	register struct service_config	*scp = SVC_CONF( sp ) ;
 	idresult_e								log_remote_user() ;
 	char										*idresult_explain() ;
@@ -203,8 +272,21 @@ void child_process( serp )
 			(void) nice( SC_NICE( scp ) ) ;
 	}
 
+	/* this is where the server gets executed  -bbraun */
 	if ( ! SC_IS_INTERNAL( scp ) )
-		exec_server( serp ) ;
+	{
+		if( scp->sc_redir_addr != NULL )
+		{
+			redir_handler( serp );
+		}
+		else
+		{
+#if defined(HAVE_SETENV)
+			setenv("REMOTE_HOST", inet_ntoa(cp->co_remote_address.sin_addr), 0);
+#endif
+			exec_server( serp ) ;
+		}
+	}
 	else
 	{
 		char name[ 180 ] ;
@@ -251,14 +333,22 @@ void child_exit()
 		register int pid ;
 		struct server *serp ;
 		
+#ifdef HAVE_WAITPID
+		pid = waitpid( -1, &status, WNOHANG ) ;
+#else
 #if defined( sun ) && defined( lint )
 		pid = wait3( (union wait *)&status, WNOHANG, RUSAGE_NULL ) ;
 #else
 		pid = wait3( &status, WNOHANG, RUSAGE_NULL ) ;
+#endif
 #endif
 
 		if ( debug.on )
+#ifdef HAVE_WAITPID
+			msg( LOG_DEBUG, func, "waitpid returned = %d", pid ) ;
+#else
 			msg( LOG_DEBUG, func, "wait3 returned = %d", pid ) ;
+#endif
 		
 		if ( pid == -1 )
 			if ( errno == EINTR )
