@@ -5,24 +5,27 @@
  */
 
 #include <sys/types.h>
-#include <sys/param.h>
 #include <syslog.h>
-#include <netdb.h>
 #include <string.h>
-
-#if defined(BSD) || defined(linux)
 #include <netinet/in.h>
-#endif
 
-#if defined(BSD)
+#ifdef HAVE_RPC_RPC_H
 #include <rpc/rpc.h>
-#else
-char *malloc() ;
 #endif
 
+#ifdef HAVE_RPC_RPCENT_H
+#include <rpc/rpcent.h>
+#endif
+
+#ifdef HAVE_NETDB_H
+#include <netdb.h>
+#endif
+
+#ifdef notdef
 void endservent() ;
 void endprotoent() ;
 void endrpcent() ;
+#endif
 
 #include "pset.h"
 #include "misc.h"
@@ -54,6 +57,15 @@ PRIVATE status_e fix_server_argv( scp )
 	char *server_name ;
 	char *func = "fix_server_argv" ;
 
+	if( SC_NAMEINARGS( scp ) ) {
+		if( !SC_SPECIFIED(scp, A_SERVER_ARGS ) ){
+			msg(LOG_ERR, func, "Must specify server args if using NAMEINARGS flag\n");
+			return( FAILED );
+		}
+
+		return(OK);
+	}
+
 	/*
 	 * Check if the user specified any server arguments.
 	 * If not, then the server_argv has not been allocated yet,
@@ -68,6 +80,7 @@ PRIVATE status_e fix_server_argv( scp )
 			out_of_memory( func ) ;
 			return( FAILED ) ;
 		}
+		scp->sc_server_argv[ 0 ] = NULL ;
 		scp->sc_server_argv[ 1 ] = NULL ;
 		SC_PRESENT( scp, A_SERVER_ARGS ) ;
 	}
@@ -123,6 +136,17 @@ PRIVATE status_e attr_fill( scp, def )
 		SC_PRESENT( scp, A_INSTANCES ) ;
 	}
 
+	if ( ! SC_SPECIFIED( scp, A_PER_SOURCE ) )
+	{
+		scp->sc_per_source = SC_SPECIFIED( def, A_PER_SOURCE ) ? def->sc_per_source : DEFAULT_INSTANCE_LIMIT ;
+		SC_PRESENT( scp, A_PER_SOURCE ) ;
+	}
+
+	if ( ! SC_SPECIFIED( scp, A_MAX_LOAD ) ) {
+		scp->sc_max_load = SC_SPECIFIED( def, A_MAX_LOAD ) ? def->sc_max_load : 0 ;
+		SC_PRESENT( scp, A_MAX_LOAD ) ;
+	}
+
 	if ( USE_DEFAULT( scp, def, A_LOG_ON_SUCCESS ) )
 	{
 		scp->sc_log_on_success = def->sc_log_on_success ;
@@ -236,21 +260,43 @@ PRIVATE status_e attr_check( scp )
 	 *			This is why we use the sc_attributes_specified mask instead
 	 *			of the sc_attributes_present mask.
 	 */
+
 	M_AND( necessary_and_specified,
 						scp->sc_specified_attributes, must_specify ) ;
 	M_XOR( necessary_and_missing, necessary_and_specified, must_specify ) ;
 
-	if ( M_ARE_ALL_CLEAR( necessary_and_missing ) )
+	if ( M_ARE_ALL_CLEAR( necessary_and_missing) )
+	{
 		return( OK ) ;
-	
+	}
+
 	/*
 	 * Print names of missing attributes
 	 */
 	for ( attr_id = 0 ; attr_id < SERVICE_ATTRIBUTES ; attr_id++ )
 		if ( M_IS_SET( necessary_and_missing, attr_id ) && 
 						( attr_name = attr_name_lookup( attr_id ) ) != NULL )
+		{
+			/* This is here to make redir without server a valid option */
+			/* We must put something in server or things get messy */
+			if( attr_id == A_SERVER )
+			{
+				if( (scp->sc_redir_addr != NULL) )
+				{
+					char *server;
+#define DEFAULT_SERVER "/bin/true"
+					server = malloc(sizeof(DEFAULT_SERVER));
+					strcpy(server, DEFAULT_SERVER);
+
+					scp->sc_server = make_string( 1, server );
+					SC_PRESENT(scp, A_SERVER);
+				
+					return OK;
+				}
+			}
 			msg( LOG_ERR, func,
 				"Service %s missing attribute %s", scp->sc_id, attr_name ) ;
+		}
 	return( FAILED ) ;
 }
 
@@ -284,13 +330,13 @@ PRIVATE status_e check_entry( scp )
 		{
 			msg( LOG_ERR, func,
 				"Internal services cannot be intercepted: %s ", scp->sc_id ) ;
-			M_CLEAR( scp->sc_flags, SF_INTERCEPT ) ;
+			M_CLEAR( scp->sc_xflags, SF_INTERCEPT ) ;
 		}
 		if ( scp->sc_wait == NO )
 		{
 			msg( LOG_ERR, func,
 				"Multi-threaded services cannot be intercepted: %s", scp->sc_id ) ;
-			M_CLEAR( scp->sc_flags, SF_INTERCEPT ) ;
+			M_CLEAR( scp->sc_xflags, SF_INTERCEPT ) ;
 		}
 	}
 
@@ -307,10 +353,11 @@ PRIVATE status_e check_entry( scp )
 	if ( SC_IS_UNLISTED( scp ) )
 		return( OK ) ;
 
-#ifndef NO_RPC
+/* #ifndef NO_RPC */
+#if defined(HAVE_RPC_RPCENT_H) || defined(HAVE_NETDB_H)
 	if ( SC_IS_RPC( scp ) )
 	{
-		struct rpcent *rep = getrpcbyname( scp->sc_name ) ;
+		struct rpcent *rep = (struct rpcent *)getrpcbyname( scp->sc_name ) ;
 
 		if ( rep == NULL )
 		{
@@ -325,6 +372,9 @@ PRIVATE status_e check_entry( scp )
 		struct servent *sep ;
 		unsigned short service_port ;
 
+		if( SC_SPECIFIED( scp, A_PROTOCOL) && SC_SPECIFIED( scp, A_PORT ) )
+			return(OK);
+		
 		/*
 		 * Check if a protocol was specified.
 		 * If so, verify it is a proper protocol for the given service.
@@ -334,7 +384,7 @@ PRIVATE status_e check_entry( scp )
 		if ( SC_SPECIFIED( scp, A_PROTOCOL ) )
 		{
 			sep = getservbyname( scp->sc_name, scp->sc_protocol.name ) ;
-			if ( sep == NULL )
+			if ( (sep == NULL) )
 			{
 				msg( LOG_ERR, func, 
 					"service/protocol combination not in /etc/services: %s/%s",
@@ -400,7 +450,7 @@ static timer_h conf_timer ;
 
 status_e create_conf_timer()
 {
-	conf_timer = timer_create( TIMER_REAL, TIMER_RETURN_ERROR, INT_NULL ) ;
+	conf_timer = xtimer_create( TIMER_REAL, TIMER_RETURN_ERROR, INT_NULL ) ;
 	return( ( conf_timer == NULL ) ? FAILED : OK ) ;
 }
 
@@ -439,7 +489,7 @@ PRIVATE status_e get_conf( fd, confp, timeout )
 		ta.ta_flags				= TIMER_LONGJMP ;
 		ta.ta_func				= NULL ;
 		ta.ta_arg				= VOID_NULL ;
-		if ( timer_start( conf_timer, &itv, TIMER_RELATIVE, &ta ) == TIMER_ERR )
+		if ( xtimer_start( conf_timer, &itv, TIMER_RELATIVE, &ta ) == TIMER_ERR )
 		{
 			msg( LOG_ERR, func, "timer creation failed" ) ;
 			return( FAILED ) ;
@@ -449,7 +499,7 @@ PRIVATE status_e get_conf( fd, confp, timeout )
 	parse_conf_file( fd, confp ) ;
 
 	if ( timeout )
-		timer_stop( conf_timer ) ;
+		xtimer_stop( conf_timer ) ;
 	parse_end() ;
 	return( OK ) ;
 
