@@ -8,7 +8,7 @@
 #define CONNECTION_H
 
 /*
- * $Id: connection.h,v 6.3 1993/06/01 22:53:50 panos Exp $
+ * $Id: connection.h,v 1.2 1999/10/15 05:18:39 bbraun Exp $
  */
 
 #include <sys/types.h>
@@ -71,10 +71,10 @@ typedef struct connection connection_s ;
 
 char *inet_ntoa() ;
 
-#define conn_addrstr( cp )																	\
-								M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS )			\
-									? inet_ntoa( (cp)->co_remote_address.sin_addr )	\
-									: "<no address>"
+#define conn_addrstr( cp )		\
+	M_IS_SET( (cp)->co_flags, COF_HAVE_ADDRESS ) \
+		? xntoa( (cp)->co_remote_address.sin_addr.s_addr )	\
+		: "<no address>"
 
 status_e 		conn_init() ;
 connection_s 	*conn_new() ;
