@@ -4,13 +4,11 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: ident.c,v 1.4 1996/01/24 19:29:19 chuck Exp $" ;
+static char RCSid[] = "$Id: ident.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#if defined(BSD) || defined(linux)
 #include <netinet/in.h>
-#endif
 #include <netdb.h>
 #include <signal.h>
 #include <syslog.h>
@@ -26,6 +24,9 @@ static char RCSid[] = "$Id: ident.c,v 1.4 1996/01/24 19:29:19 chuck Exp $" ;
 #include "sconst.h"
 #include "server.h"
 
+PRIVATE char *get_line( int sd, register char *buf, unsigned bufsize );
+PRIVATE char *verify_line( char *line, unsigned local_port, unsigned remote_port );
+
 void msg() ;
 status_e write_buf() ;
 
@@ -65,8 +66,6 @@ idresult_e log_remote_user( serp, timeout )
 	int						sin_len ;
 	char						*p ;
 	char						*func = "log_remote_user" ;
-	PRIVATE char			*get_line() ;
-	PRIVATE char			*verify_line() ;
 
 #ifndef NO_TIMERS
 	/*
@@ -75,7 +74,7 @@ idresult_e log_remote_user( serp, timeout )
 	STOP_TIMER() ;
 #endif
 
-	if ( timeout && (int) signal( SIGALRM, sigalrm_handler ) == -1 )
+	if ( timeout && signal( SIGALRM, sigalrm_handler ) == SIG_ERR )
 	{
 		msg( LOG_ERR, func, "signal: %m" ) ;
 		return( IDR_ERROR ) ;
