@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: int.c,v 1.2 1995/09/10 15:42:38 chuck Exp $" ;
+static char RCSid[] = "$Id: int.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -14,12 +14,13 @@ static char RCSid[] = "$Id: int.c,v 1.2 1995/09/10 15:42:38 chuck Exp $" ;
 
 #include "config.h"
 #include "connection.h"
-#ifdef BSD
 #include "service.h"
-#endif
 #include "server.h"
 #include "int.h"
 
+PRIVATE void start_server( struct intercept *ip );
+PRIVATE void terminate_server( struct intercept *ip );
+
 void msg() ;
 char *inet_ntoa() ;
 unsigned long inet_addr() ;
@@ -73,8 +74,6 @@ void intercept( serp )
 {
 	struct service *sp = SERVER_SERVICE( serp ) ;
 	initfunc initializer ;
-	PRIVATE void start_server() ;
-	PRIVATE void terminate_server() ;
 
 #ifdef DEBUG_INTERCEPTOR
 	if ( debug.on )
