@@ -8,13 +8,11 @@
 #define INT_H
 
 /*
- * $Id: int.h,v 1.2 1995/09/10 15:42:38 chuck Exp $
+ * $Id: int.h,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $
  */
 
 #include <sys/types.h>
-#ifdef BSD
 #include <netinet/in.h>
-#endif
 
 #include "pset.h"
 
