@@ -4,14 +4,11 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: log.c,v 1.7 1996/01/24 19:29:19 chuck Exp $" ;
+static char RCSid[] = "$Id: log.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#include <sys/param.h>
-#ifndef BSD
 #include <sys/wait.h>
-#endif
 #include <syslog.h>
 #include <time.h>
 
@@ -24,6 +21,11 @@ static char RCSid[] = "$Id: log.c,v 1.7 1996/01/24 19:29:19 chuck Exp $" ;
 #include "sconst.h"
 #include "service.h"
 #include "server.h"
+#include "config.h"
+
+#if !defined(HAVE_SNPRINTF)
+extern int      snprintf(char *, size_t, const char *, ...);
+#endif
 
 
 void msg() ;
@@ -33,6 +35,29 @@ void msg() ;
 
 time_t time() ;
 
+struct xaddr
+{
+	unsigned char a;
+	unsigned char b;
+	unsigned char c;
+	unsigned char d;
+};
+
+char *xntoa(unsigned long inadr)
+{
+	char *tmp;
+	struct xaddr foo;
+
+	bcopy(&inadr, &foo, 4);
+	tmp = (char *)malloc(16);
+
+	snprintf(tmp, 16, "%d.%d.%d.%d", foo.a, foo.b, foo.c, foo.d);
+
+	return tmp;
+
+	
+}
+
 
 PRIVATE int log_common() ;
 
@@ -51,6 +76,7 @@ void svc_log_success( sp, cp, pid )
 	register struct service_config	*scp = SVC_CONF( sp ) ;
 	register int							len ;
 	register int							cc ;
+	char *foo;
 
 	if ( ! SVC_LOGS_ON_SUCCESS( sp ) )
 		return ;
@@ -219,11 +245,7 @@ void svc_log_exit( sp, serp )
  * going through the proper channels (i.e. log_{success,failure} and log_exit)
  */
 /* VARARGS3 */
-void svc_logprint( sp, line_id, fmt, va_alist )
-	register struct service	*sp ;
-	char							*line_id ;
-	char							*fmt ;
-	va_dcl
+void svc_logprint( register struct service *sp, char *line_id, char *fmt, ...)
 {
 	char		buf[ LOGBUF_SIZE ] ;
 	int		bufsize = sizeof( buf ) ;
@@ -235,7 +257,7 @@ void svc_logprint( sp, line_id, fmt, va_alist )
 		return ;
 
 	len = strx_nprint( buf, bufsize, "%s: %s ", line_id, SVC_ID( sp ) ) ;
-	va_start( ap ) ;
+	va_start( ap, fmt ) ;
 	cc = strx_nprintv( &buf[ len ], bufsize, fmt, ap ) ;
 	va_end( ap ) ;
 	xlog_write( sp->svc_log, buf, len+cc, XLOG_NO_SIZECHECK ) ;
