@@ -4,22 +4,24 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: parsers.c,v 1.3 1995/09/10 18:41:13 chuck Exp $" ;
+static char RCSid[] = "$Id: parsers.c,v 1.5 1999/10/14 23:46:27 bbraun Exp $" ;
 
-#ifdef linux
-#include <unistd.h>
-#endif
 #include <sys/file.h>
 #include <syslog.h>
 #include <string.h>
 #include <netdb.h>
 #include <pwd.h>
 #include <grp.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <unistd.h>
 
-#ifdef BIND_IF
-#include <netinet/in.h>  /* NV */
+#if defined(hpux) && !defined(X_OK)
+#define X_OK 1
 #endif
 
+
 int free() ;
 
 #include "pset.h"
@@ -31,14 +33,13 @@ int free() ;
 #include "config.h"
 #include "addr.h"
 
-#define NEW_SET( set, v1, v2 )												\
-				if ( (set) == NULL && 										\
-					( (set) = pset_create( (set), (v1), (v2) ) ) == NULL )	\
-				{															\
-					out_of_memory( func ) ;									\
-					return( FAILED ) ;										\
-				}
-
+#define NEW_SET( set, v1, v2 )                 \
+	if ( (set) == NULL && \
+	( (set) = pset_create( (set), (v1), (v2) ) ) == NULL )	\
+	{	\
+		out_of_memory( func ) ;	\
+		return( FAILED ) ;	\
+	}
 
 void msg() ;
 void parsemsg() ;
@@ -105,7 +106,7 @@ status_e flags_parser( values, scp, op )
 	enum assign_op op ;
 {
 	return( parse_value_list( values,
-						&scp->sc_flags, service_flags, PLUS_EQ, "service flag" ) ) ;
+						&scp->sc_xflags, service_flags, PLUS_EQ, "service flag" ) ) ;
 }
 
 
@@ -258,7 +259,22 @@ status_e group_parser( values, scp, op )
 	return( OK ) ;
 }
 
+status_e groups_parser( values, scp, op )
+	pset_h values ;
+	struct service_config *scp ;
+	enum assign_op op ;
+{
+	char *val = (char *) pset_pointer( values, 0 ) ;
+	char *func = "groups_parser" ;
 
+	if ( EQ( val, "yes" ) )
+		scp->sc_groups = YES ;
+	else if ( EQ( val, "no" ) )
+		scp->sc_groups = NO ;
+	else
+		parsemsg( LOG_ERR, func, "Bad value for groups: %s", val ) ;
+	return( OK );
+}
 
 status_e server_parser( values, scp, op )
 	pset_h values ;
@@ -304,50 +320,45 @@ status_e server_args_parser( values, scp, op )
 	 */
 	argv = argv_alloc( n_args+1 ) ;
 	
-	for ( u = 0, argv_index = 1 ; u < pset_count( values ) ; u++, argv_index++ )
+	if( SC_NAMEINARGS( scp ) )
 	{
-		register char *s = make_string( 1, (char *) pset_pointer( values, u ) ) ;
-
-		if ( s == NULL )
+		for (u = 0; u < pset_count( values ) ; u++)
 		{
-			for ( i = 1 ; i < argv_index ; i++ )
-				free( argv[ i ] ) ;
-			free( (char *) argv ) ;
-			out_of_memory( func ) ;
-			return( FAILED ) ;
-		}
-		argv[ argv_index ] = s ;
-	}
+			register char *s = make_string(1, (char *) pset_pointer( values, u )) ;
 
-	argv[ argv_index ] = argv[ 0 ] = NULL ;
+			if ( s == NULL )
+			{
+ 				for ( i = 1 ; i < u ; i++ )
+					free( argv[ i ] ) ;
+				free( (char *) argv ) ;
+ 				out_of_memory( func ) ;
+ 				return( FAILED ) ;
+			}
+			argv[ u ] = s ;
+		}
+	}
+	else
+	{
+		for (u = 0, argv_index = 1 ; u < pset_count( values ) ; u++, argv_index++)
+		{
+			register char *s = make_string(1, (char *) pset_pointer( values, u )) ;
+
+			if ( s == NULL )
+			{
+ 				for ( i = 1 ; i < argv_index ; i++ )
+					free( argv[ i ] ) ;
+				free( (char *) argv ) ;
+ 				out_of_memory( func ) ;
+ 				return( FAILED ) ;
+			}
+			argv[ argv_index ] = s ;
+		}
+		argv[ argv_index ] = argv[ 0 ] = NULL ;
+	}
 	scp->sc_server_argv = argv ;
 	return( OK ) ;
 }
 
-#ifdef BIND_IF
-/* 
- * NV - parsing routine for interface IP address 
- */
-status_e interface_parser( values, scp, op )
-	pset_h values ;
-	struct service_config *scp ;
-	enum assign_op op ;
-{
-	char   *interface = (char *) pset_pointer( values, 0 ) ;
-	char   *func      = "interface_parser" ;
-
-    /* assign an IP address */
-	scp->sc_interface.s_addr = inet_addr( interface );
-
-	/* trap badly formed IP addresses */
-	if ( scp->sc_interface.s_addr == -1 )
-	{
-		parsemsg( LOG_ERR, func, "Badly formed interface IP address: %s", 
-                  interface ) ;
-		return( FAILED ) ;
-	}
-}
-#endif
 
 status_e instances_parser( values, scp, op )
 	pset_h values ;
@@ -372,6 +383,27 @@ status_e instances_parser( values, scp, op )
 	return( OK ) ;
 }
 
+status_e per_source_parser( values, scp, op )
+	pset_h values ;
+	struct service_config *scp ;
+	enum assign_op op ;
+{
+	char *per_source = (char *) pset_pointer( values, 0 ) ;
+	char *func = "per_source_parser" ;
+
+	if ( EQ( per_source, "UNLIMITED" ) )
+		scp->sc_per_source = UNLIMITED;
+	else
+	{
+		scp->sc_per_source = atoi( per_source ) ;
+		if ( scp->sc_per_source < 0 )
+		{
+			parsemsg( LOG_ERR, func, "Number of per source instances is negative: %s", per_source ) ;
+			return( FAILED );
+		}
+	}
+	return(OK);
+}
 
 status_e id_parser( values, scp, op )
 	pset_h values ;
@@ -827,6 +859,166 @@ status_e no_access_parser( values, scp, op )
 	return( parse_inet_addresses( values, op, &scp->sc_no_access ) ) ;
 }
 
+status_e banner_parser(pset_h values, struct service_config *scp, enum assign_op op)
+{
+	if( pset_pointer(values, 0) == NULL )
+	{
+		return FAILED;
+	}
+
+	scp->sc_banner = (char *)malloc(strlen(pset_pointer(values,0)));
+	strcpy(scp->sc_banner, pset_pointer(values,0));
+
+	return OK;
+}
+
+status_e banner_success_parser(pset_h values, struct service_config *scp, enum assign_op op)
+{
+	if( pset_pointer(values, 0) == NULL )
+		return FAILED;
+
+	scp->sc_banner_success = (char *)malloc(strlen(pset_pointer(values,0)));
+	if( scp->sc_banner_success == NULL )
+		return FAILED;
+
+	strcpy(scp->sc_banner_success, pset_pointer(values,0));
+	return OK;
+}
+
+status_e banner_fail_parser(pset_h values, struct service_config *scp, enum assign_op op)
+{
+	if( pset_pointer(values, 0) == NULL )
+		return FAILED;
+
+	scp->sc_banner_fail = (char *)malloc(strlen(pset_pointer(values,0)));
+	if( scp->sc_banner_success == NULL )
+		return FAILED;
+	
+	strcpy(scp->sc_banner_fail, pset_pointer(values,0));
+	return OK;
+}
+
+#ifdef HAVE_LOADAVG
+status_e max_load_parser(pset_h values, struct service_config *scp, enum assign_op op)
+{
+	char *func = "max_load_parser" ;
+	char *adr = (char *)pset_pointer(values, 0);
+
+	if( sscanf(adr, "%lf", &scp->sc_max_load) < 1 ) {
+		msg(LOG_ERR, func, "error reading max_load argument\n");
+		return FAILED;
+	}
+
+	if( scp->sc_max_load == 0 ) {
+		msg(LOG_ERR, func, "error parsing max_load arguement\n");
+		return FAILED;
+	}
+
+	return OK;
+}
+#endif
+
+status_e redir_parser(pset_h values, struct service_config *scp, enum assign_op op)
+{
+	char *adr = (char *)pset_pointer(values, 0);
+	char *foo;
+	char *uptmp;
+	unsigned long tempaddr;
+	struct hostent *he;
+
+	uptmp = pset_pointer(values, 1);
+
+	scp->sc_redir_port = strtol(uptmp, NULL, 10);
+
+	scp->sc_redir_addr = (struct in_addr *)malloc(sizeof(struct in_addr));
+
+	if( scp->sc_redir_addr == NULL )
+	{
+		syslog(LOG_ERR, "redir_parser: can't allocate space for redir addr\n");
+		return FAILED;
+	}
+
+	he = gethostbyname(adr);
+	if( he == NULL )
+	{
+#if defined(NO_INET_ATON)
+		tempaddr = inet_addr(adr);
+		bcopy(&tempaddr, scp->sc_redir_addr, sizeof(tempaddr));
+#else
+		if( inet_aton(adr, scp->sc_redir_addr) < 1 )
+		{
+			syslog(LOG_ERR, "redir_parser: invalid IP address %s\n", adr);
+			free(scp->sc_redir_addr);
+			return FAILED;
+		}
+#endif
+	}
+	else
+	{
+		/* gethostbyname succeeded */
+		bcopy(he->h_addr, scp->sc_redir_addr, sizeof(he->h_addr));
+	}
+
+	foo = inet_ntoa(*scp->sc_redir_addr);
+
+	if( foo == NULL )
+	{
+		syslog(LOG_ERR, "inet_ntoa failed\n");
+		free(scp->sc_redir_addr);
+		return FAILED;
+	}
+
+	return OK;
+}
+
+status_e bind_parser( pset_h values, struct service_config *scp, enum assign_op op)
+{
+	char *adr = (char *)pset_pointer(values, 0);
+	char *foo;
+	unsigned long tempaddr;
+	struct hostent *he;
+
+	scp->sc_bind_addr = (struct in_addr *)malloc(sizeof(struct in_addr));
+
+	if( scp->sc_bind_addr == NULL )
+	{
+		syslog(LOG_ERR, "bind_parser: can't allocate space for bind addr\n");
+		return FAILED;
+	}
+
+	he = gethostbyname(adr);
+	if( he == NULL )
+	{
+#if defined(NO_INET_ATON)
+		tempaddr = inet_addr(adr);
+		bcopy(&tempaddr, scp->sc_bind_addr, sizeof(tempaddr));
+#else
+		if( inet_aton(adr, scp->sc_bind_addr) < 1 )
+		{
+			syslog(LOG_ERR, "bind_parser: invalid IP address %s\n", adr);
+			free(scp->sc_bind_addr);
+			return FAILED;
+		}
+#endif
+	}
+	else
+	{
+		/* gethostbyname succeeded, so we now have the address */
+		bcopy(he->h_addr, scp->sc_bind_addr, sizeof(he->h_addr));
+	}
+
+	foo = inet_ntoa(*scp->sc_bind_addr);
+
+	if( foo == NULL )
+	{
+		syslog(LOG_ERR, "inet_ntoa failed\n");
+		free(scp->sc_bind_addr);
+		return FAILED;
+	}
+	
+	return OK;
+	
+}
 
 status_e access_times_parser( values, scp, op )
 	pset_h values ;
