@@ -4,19 +4,26 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: service.c,v 1.5 1996/01/24 19:29:19 chuck Exp $" ;
+static char RCSid[] = "$Id: service.c,v 1.3 1999/10/13 04:22:34 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#ifdef linux
-#include <sys/time.h>
-#endif
-#if defined(BSD) || defined linux
 #include <netinet/in.h>
-#endif
 #include <netdb.h>
 #include <syslog.h>
 #include <fcntl.h>
+#include <netinet/tcp.h>
+
+/* This is because of differences between glibc2 and libc5 in linux... */
+#ifndef FD_SET
+#ifdef HAVE_LINUX_TIME_H
+#include <linux/time.h>
+#endif
+#endif
+
+#ifdef HAVE_SYS_FILE_H
+#include <sys/file.h>
+#endif
 
 #include "sio.h"
 
@@ -37,9 +44,13 @@ static char RCSid[] = "$Id: service.c,v 1.5 1996/01/24 19:29:19 chuck Exp $" ;
 #define RESUME( sp )				(sp)->svc_state = SVC_ACTIVE
 #define DISABLE( sp )			(sp)->svc_state = SVC_DISABLED
 
+
 void msg() ;
 void out_of_memory() ;
 
+PRIVATE status_e svc_generic_handler( struct service *sp, connection_s *cp );
+PRIVATE void deactivate( struct service *sp );
+
 time_t time() ;
 char *malloc() ;
 
@@ -106,6 +117,7 @@ void svc_free( sp )
 	FREE_SVC( sp ) ;
 }
 
+
 /*
  * Currently, address control works by looking at the fields 
  * only_from/no_access of struct service.
@@ -176,32 +188,24 @@ PRIVATE status_e set_fd_modes( sp )
 PRIVATE status_e activate_rpc( sp )
    register struct service *sp ;
 {
-	struct sockaddr_in		sin ;
-	int						sin_len ;
-	unsigned long			vers ;
-	struct service_config	*scp						= SVC_CONF( sp ) ;
-	struct rpc_data			*rdp						= SC_RPCDATA( scp ) ;
-	char					*sid						= SC_ID( scp ) ;
-	unsigned				registered_versions			= 0 ;
-	int						sd							= SVC_FD( sp ) ;
-	char					*func						= "activate_rpc" ;
+	struct sockaddr_in     sin ;
+	int                    sin_len ;
+	unsigned long          vers ;
+	struct service_config *scp = SVC_CONF( sp ) ;
+	struct rpc_data       *rdp = SC_RPCDATA( scp ) ;
+	char                  *sid = SC_ID( scp ) ;
+	unsigned               registered_versions = 0 ;
+	int                    sd = SVC_FD( sp ) ;
+	char                  *func = "activate_rpc" ;
 
 	CLEAR( sin ) ;
 
-    /*
-     * N Verdon (5/6/95) changed the sin.sin_addr.s_addr to
-     * use the variable scp->sc_interface.s_addr to allow
-     * binding to a specific IP address.
-     */
-	sin.sin_family      = AF_INET ;
-
-#ifdef BIND_IF
-	sin.sin_addr.s_addr = scp->sc_interface.s_addr;
-#else
-	sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
-#endif
-
-	sin.sin_port        = 0 ; /* let the system give us a port */
+	sin.sin_family = AF_INET ;
+	if( scp->sc_bind_addr != NULL )
+		sin.sin_addr.s_addr = scp->sc_bind_addr->s_addr ;
+	else
+		sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
+	sin.sin_port = 0 ;         /* let the system give us a port */
 
 	if ( bind( sd, SA( &sin ), sizeof( sin ) ) == -1 )
 	{
@@ -244,41 +248,38 @@ PRIVATE status_e activate_rpc( sp )
 
 
 PRIVATE status_e activate_normal( sp )
-	register struct service *sp ;
+   register struct service *sp ;
 {
 	struct sockaddr_in		sin ;
-	int						sd				= SVC_FD( sp ) ;
-	struct service_config	*scp			= SVC_CONF( sp ) ;
-	unsigned short			service_port	= SC_PORT( scp ) ;
-	char					*sid			= SC_ID( scp ) ;
-	char					*func			= "activate_normal" ;
+	int							sd					= SVC_FD( sp ) ;
+	struct service_config	*scp				= SVC_CONF( sp ) ;
+	unsigned short				service_port	= SC_PORT( scp ) ;
+	char							*sid				= SC_ID( scp ) ;
+	char							*func				= "activate_normal" ;
 
 	CLEAR( sin ) ;
 
-	/*
-	 * N Verdon (5/6/95) changed the sin.sin_addr.s_addr to
-     * use the variable scp->sc_interface.s_addr to allow
-     * binding to a specific IP address.
-	 */
-	sin.sin_family      = AF_INET ;
-#ifdef BIND_IF
-	sin.sin_addr.s_addr = scp->sc_interface.s_addr;
-#else
-	sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
-#endif
-	sin.sin_port        = htons( service_port ) ;
+	sin.sin_family = AF_INET ;
+	if( scp->sc_bind_addr != NULL )
+		sin.sin_addr.s_addr = scp->sc_bind_addr->s_addr ;
+	else
+		sin.sin_addr.s_addr = htonl( INADDR_ANY ) ;
+	sin.sin_port = htons( service_port ) ;
 
 	if ( reuse_option || SC_REUSE_ADDRESS( scp ) )
 	{
 		int on = 1 ;
 
-		if ( setsockopt( sd, SOL_SOCKET, SO_REUSEADDR, 
-                         (char *) &on, sizeof( on ) ) == -1 
-           )
-		{
-			msg( LOG_WARNING, func, 
-                 "setsockopt SO_REUSEADDR failed (%m). service = %s", sid ) ;
-		}
+		if ( setsockopt( sd, SOL_SOCKET, SO_REUSEADDR, (char *) &on, sizeof( on ) ) == -1 )
+			msg( LOG_WARNING, func, "setsockopt SO_REUSEADDR failed (%m). service = %s", sid ) ;
+	}
+
+	if( SC_NODELAY( scp ) && (scp->sc_protocol.value == IPPROTO_TCP) )
+	{
+		int on = 1;
+
+		if ( setsockopt( sd, IPPROTO_TCP, TCP_NODELAY, (char *) &on, sizeof(on) ) == -1 )
+			msg( LOG_WARNING, func, "setsockopt TCP_NODELAY failed (%m). service = %s", sid ) ;
 	}
 
 	if ( bind( sd, SA( &sin ), sizeof( sin ) ) == -1 )
@@ -286,7 +287,6 @@ PRIVATE status_e activate_normal( sp )
 		msg( LOG_ERR, func, "bind failed (%m). service = %s", sid ) ;
 		return( FAILED ) ;
 	}
-
 	return( OK ) ;
 }
 
@@ -301,8 +301,6 @@ status_e svc_activate( sp )
 	status_e						status ;
    char							*func = "svc_activate" ;
 	voidfunc						get_shutdown_by_name() ;
-	PRIVATE status_e			svc_generic_handler() ;
-	PRIVATE void				deactivate() ;
 
 	sp->svc_fd = socket( AF_INET, 
 								SC_SOCKET_TYPE( scp ), SC_PROTOVAL( scp ) ) ;
@@ -629,11 +627,38 @@ PRIVATE status_e svc_generic_handler( sp, cp )
 }
 
 
+#define TMPSIZE 1024
 status_e svc_access_control( sp, cp )
 	register struct service		*sp ;
 	connection_s					*cp ;
 {
 	access_e result ;
+	char *func = "svc_access_control";
+	struct service_config *scp = SVC_CONF( sp ) ;
+
+	/* print the banner regardless of access control */
+	if ( scp->sc_banner != NULL ) {
+		char tmpbuf[TMPSIZE];
+		int retval;
+		int bannerfd = open(scp->sc_banner, O_RDONLY);
+
+		if( bannerfd < 0 ) {
+			msg( LOG_ERR, func, "service = %s, open of banner %s failed\n", SVC_ID( sp ), scp->sc_banner);
+			return FAILED;
+		}
+
+		bzero(tmpbuf, TMPSIZE);
+		while( retval = read(bannerfd, tmpbuf, TMPSIZE) ) {
+			Sprint(cp->co_descriptor, "%s\n", tmpbuf);
+			Sflush();
+			if( retval < TMPSIZE )
+				break;
+
+			bzero(tmpbuf, TMPSIZE);
+		}
+
+		close(bannerfd);
+	}
 
 	result = access_control( sp, cp, MASK_NULL ) ;
 	if ( result != AC_OK )
@@ -672,8 +697,61 @@ status_e svc_access_control( sp, cp )
 
 		if ( report_failure )
 			svc_log_failure( sp, cp, result ) ;
+
+		if ( scp->sc_banner_fail != NULL )
+		{
+			char tmpbuf[TMPSIZE];
+			int retval;
+			int bannerfd = open(scp->sc_banner_fail, O_RDONLY);
+	
+			if( bannerfd < 0 )
+			{
+				msg( LOG_ERR, func, "service = %s, open of banner %s failed\n", 
+					SVC_ID( sp ), scp->sc_banner_fail);
+				return FAILED;
+			}
+	
+			bzero(tmpbuf, TMPSIZE);
+			while( retval = read(bannerfd, tmpbuf, TMPSIZE) )
+			{
+				Sprint(cp->co_descriptor, "%s\n", tmpbuf);
+				Sflush();
+
+				if( retval < TMPSIZE )
+					break;
+				bzero(tmpbuf, TMPSIZE);
+			}
+
+			close(bannerfd);
+		}
+
 		return( FAILED ) ;
 	}
+
+	/* print the access granted banner */
+	if ( scp->sc_banner_success != NULL ) {
+		char tmpbuf[TMPSIZE];
+		int retval;
+		int bannerfd = open(scp->sc_banner_success, O_RDONLY);
+	
+		if( bannerfd < 0 ) {
+			msg( LOG_ERR, func, "service = %s, open of banner %s failed\n", SVC_ID( sp ), scp->sc_banner_success);
+			return FAILED;
+		}
+	
+		bzero(tmpbuf, TMPSIZE);
+		while( retval = read(bannerfd, tmpbuf, TMPSIZE) ) {
+			Sprint(cp->co_descriptor, "%s\n", tmpbuf);
+			Sflush();
+
+			if( retval < TMPSIZE )
+				break;
+			bzero(tmpbuf, TMPSIZE);
+		}
+
+		close(bannerfd);
+	}
+
 	return( OK ) ;
 }
 
