@@ -11,7 +11,7 @@
 #include <netinet/in.h>
 
 /*
- * $Id: service.h,v 1.2 1995/09/10 14:26:36 chuck Exp $
+ * $Id: service.h,v 1.2 1999/10/13 04:22:34 bbraun Exp $
  */
 
 #include "xlog.h"
@@ -19,17 +19,16 @@
 #include "defs.h"
 #include "sconf.h"
 
-
 /*
  * NOTE: A service can be disabled but not deleted if it has any servers
  *       running
  */
 typedef enum                     /* service states */
    {
-      SVC_NOT_STARTED = 0,      /* no attempt to start it yet       */
-      SVC_ACTIVE,               /* service is available             */
-	  SVC_SUSPENDED,			/* service is suspended					*/
-      SVC_DISABLED              /* service disabled                 */
+      SVC_NOT_STARTED = 0,       /* no attempt to start it yet       */
+      SVC_ACTIVE,                /* service is available             */
+      SVC_SUSPENDED,             /* service is suspended             */
+      SVC_DISABLED               /* service disabled                 */
    } state_e ;
 
 
@@ -38,29 +37,22 @@ typedef enum                     /* service states */
  */
 struct service
 {
-	state_e		svc_state ;
-
-	int 			svc_ref_count ;			/* number of pointers to this struct */
-
-	struct service_config *svc_conf ;	/* service configuration				 */
-
-	int			svc_fd ;
-
-	unsigned 	svc_running_servers ;
-	unsigned 	svc_retry_servers ;
-
-	unsigned 	svc_attempts ;			/* # of attempts to start a server     */
-	time_t		svc_start_time ;		/* since this time							*/
-
+	state_e                svc_state ;
+	int                    svc_ref_count ;	/* # of pters to this struct */
+	struct service_config *svc_conf ;	/* service configuration */
+	int                    svc_fd ;
+	unsigned               svc_running_servers ;
+	unsigned               svc_retry_servers ;
+	unsigned               svc_attempts ; /* # of attempts to start server*/
+	time_t                 svc_start_time ;	/* since this time */
 	/*
-	 * 	svc_shutdown:		invoked to playback the protocol and get information
-	 *								from the other end. It also sets up the connection
-	 *								for an orderly close.
-	 *		svc_handler:		invoked to handle new requests
+	 * svc_shutdown: invoked to playback the protocol and get information
+	 *		from the other end. It also sets up the connection
+	 *		for an orderly close.
+	 * svc_handler:	invoked to handle new requests
 	 */
-	voidfunc		svc_shutdown_func ;	/* ARGS: int sd, char **msgp           */
-	statfunc		svc_handler_func ;	/* ARGS: service *, connection *       */
-
+	voidfunc               svc_shutdown_func ; /* ARGS:int sd, char**msgp */
+	statfunc               svc_handler_func ;/*ARGS:service *,connection **/
 	/*
 	 * These fields are used for access control; they either point to the
 	 * service access control lists or to the default access control lists
@@ -72,17 +64,9 @@ struct service
 	 * These fields are used to avoid generating too many messages when
 	 * receiving datagrams from a bad address.
 	 */
-	struct sockaddr_in	svc_last_dgram_addr ;
-	time_t					svc_last_dgram_time ;
-
-	xlog_h		svc_log ;
-
-#ifdef BIND_IF
-    /*
-     * N Verdon (5/6/95) - interface to bring up this service on
-     */
-    struct in_addr	svc_interface;
-#endif
+	struct sockaddr_in                      svc_last_dgram_addr ;
+	time_t                                  svc_last_dgram_time ;
+	xlog_h                                  svc_log ;
 } ;
 
 #define SP( p )						( (struct service *) (p) )
@@ -104,10 +88,6 @@ struct service
 #define SVC_IS_AVAILABLE( sp )	( SVC_IS_ACTIVE(sp) || SVC_IS_SUSPENDED(sp) )
 #define SVC_IS_DISABLED( sp )		( (sp)->svc_state == SVC_DISABLED )
 
-#ifdef BIND_IF
-#define SVC_INTERFACE( sp )		( (sp)->svc_interface )
-#endif
-
 /*
  * Predicate checking macros
  */
@@ -176,7 +156,7 @@ void					svc_shutdown() ;
 void					svc_log_success() ;
 void					svc_log_failure() ;
 void					svc_log_exit() ;
-void					svc_logprint() ;
+void					svc_logprint(register struct service *, char *, char *, ...) ;
 void					svc_postmortem() ;
 
 #endif	/* SERVICE_H */
