@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: signals.c,v 1.7 1995/09/11 18:18:21 chuck Exp $" ;
+static char RCSid[] = "$Id: signals.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/time.h>
@@ -20,10 +20,12 @@ static char RCSid[] = "$Id: signals.c,v 1.7 1995/09/11 18:18:21 chuck Exp $" ;
 #include "config.h"
 #include "flags.h"
 
-extern int errno ;
-
 void msg() ;
 
+PRIVATE void my_handler( int sig );
+
+PRIVATE void general_handler( int sig );
+
 void exit() ;
 time_t time() ;
 
@@ -161,9 +163,6 @@ PRIVATE status_e handle_signal( sig )
 {
 	struct sigaction		sa ;
 	voidfunc					sig_handler ;
-	PRIVATE void			my_handler() ;				/* for the signals we are 	*/
-																/* interested in				*/
-	PRIVATE void			general_handler() ;		/* for everything else		*/
 
 	switch ( sig )
 	{
@@ -183,7 +182,6 @@ PRIVATE status_e handle_signal( sig )
 		case SIGTTIN:
 		case SIGTTOU:
 		case SIGTSTP:
-		case SIGCONT:
 			if ( debug.on )
 				return( OK ) ;
 			/* FALL THROUGH */
@@ -341,58 +339,20 @@ PRIVATE void bad_signal()
 	}
 }
 
-
-#if defined( sun ) && defined( sparc )
-#undef SP
-
-#include <machine/reg.h>
-#include <machine/frame.h>
-
-PRIVATE void stack_trace( scp )
-	struct sigcontext *scp ;
-{
-	struct frame		*fp ;
-	struct rwindow		*rwp ;
-	char					tracebuf[ 1000 ] ;
-	int					len	= 0 ;
-	unsigned				size	= sizeof( tracebuf ) - 1 ;
-
-	tracebuf[ size ] = NUL ;
-
-	if ( scp->sc_wbcnt != 0 )
-		return ;
-	rwp = (struct rwindow *) scp->sc_sp ;
-	len = strx_nprint( tracebuf, size, "%#x %#x", scp->sc_pc, rwp->rw_rtn ) ;
-	size -= len ;
-
-	for ( fp = (struct frame *) rwp->rw_fp ; fp != NULL ; fp = fp->fr_savfp ) 
-	{
-		int cc ;
-
-		cc = strx_nprint( &tracebuf[ len ], size,
-												" %#x", (unsigned) fp->fr_savpc ) ;
-		len += cc ;
-		size -= cc ;
-		fp = fp->fr_savfp ;
-	}
-
-	msg( LOG_CRIT, "stack_trace", "%s", tracebuf ) ;
-}
-#endif	/* sun && sparc */
-
-
-
 char *sig_name( sig )
 	int sig ;
 {
 	static char signame_buf[ 30 ] ;
 
+#if defined(sun)
+#define NO_SIGLIST
+#endif
+
+#if defined(hpux)
+#define NO_SIGLIST
+#endif
+
 #if !defined( NO_SIGLIST ) && defined( NSIG )
-
-#if !defined(linux) && !(BSD > 199300) && !defined(__FreeBSD__) && !defined(__NetBSD__)
-	extern char *sys_siglist[] ;
-#endif	/* linux */
-
 	if ( sig < NSIG )
 		return( strx_sprint( signame_buf, sizeof( signame_buf ),
 							"%d (%s)", sig, sys_siglist[ sig ] ) ) ;
@@ -402,49 +362,16 @@ char *sig_name( sig )
 
 
 /*
- * For SIGSEGV and SIGBUS we invoke the bad_signal() function (if this is 
- * SunOS, we log the address where the problem occured).
- *
- * If we are not running under SunOS, we use the NO_SIGVEC flag to determine 
- * if we have a struct sigcontext (since struct sigcontext is documented in 
- * the sigvec man page).
+ * For SIGSEGV and SIGBUS we invoke the bad_signal() function
  *
  * For other signals, we just log the fact that they occured.
  * SIGINT is a special case since in debug.on mode, it will 
  * cause termination.
  */
 
-#ifdef sun
-
-PRIVATE void general_handler( sig, code, scp, addr )
-	int sig, code ;
-	struct sigcontext *scp;
-	char *addr;
-{
-	int pc = scp->sc_pc ;
-
-#else		/* not a Sun */
-
-#if !defined(NO_SIGVEC) && !defined(linux)	/* this is weird */
-
-PRIVATE void general_handler( sig, code, scp )
-	int sig, code ;
-	struct sigcontext *scp;
-{
-	char *addr = NULL ;
-	int pc = scp->sc_pc ;
-
-#else		/* defined( NO_SIGVEC ) */
-
 PRIVATE void general_handler( sig )
 	int sig ;
 {
-	char *addr = NULL ;
-	int pc = 0 ;
-
-#endif	/* ! NO_SIGVEC */
-#endif	/* sun */
-
 	sigset_t badsigs ;
 	char *func = "general_handler" ;
 
@@ -459,18 +386,12 @@ PRIVATE void general_handler( sig )
 	{
 		case SIGBUS:
 		case SIGSEGV:
-			msg( LOG_CRIT, func,
-					"(%d) Unexpected signal: %s, pc = %#x, address = %#x.",
-						getpid(), sig_name( sig ), pc, addr ) ;
-#if defined( sun ) && defined( sparc )
-			stack_trace( scp ) ;
-#endif
+			msg( LOG_CRIT, func, "(%d) Unexpected signal: %s", getpid(), sig_name( sig ) ) ;
 			bad_signal() ;
 			break ;
 		
 		default:
-			msg( LOG_NOTICE, func, "Unexpected signal %s (addr=%#x)",
-						sig_name( sig ), addr ) ;
+			msg( LOG_NOTICE, func, "Unexpected signal %s", sig_name( sig ) ) ;
 			if ( debug.on && sig == SIGINT )
 				exit( 1 ) ;
 	}
@@ -537,10 +458,10 @@ void signal_default_state()
 
 	for ( sig = 1 ; sig < nsig ; sig++ )
 		if ( sigismember( &reset_sigs, sig ) == 1 )
-			if ( (int) signal( sig, SIG_DFL ) == -1 )
+			if ( signal( sig, SIG_DFL ) == SIG_ERR )
 			{
 				msg( LOG_ERR, "reset_signals",
-					"signal(3) failed for signal %s: %m", sig_name( sig ) ) ;
+					"signal failed for signal %s: %m", sig_name( sig ) ) ;
 				if ( debug.on )
 					_exit( 1 ) ;
 			}
