@@ -4,7 +4,7 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: tcpint.c,v 6.6 1993/06/15 23:25:57 panos Exp $" ;
+static char RCSid[] = "$Id: tcpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -13,6 +13,10 @@ static char RCSid[] = "$Id: tcpint.c,v 6.6 1993/06/15 23:25:57 panos Exp $" ;
 #include <signal.h>
 #include <errno.h>
 
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+
 #include "config.h"
 #include "int.h"
 #include "access.h"
@@ -21,6 +25,7 @@ static char RCSid[] = "$Id: tcpint.c,v 6.6 1993/06/15 23:25:57 panos Exp $" ;
 char *inet_ntoa() ;
 void msg() ;
 
+
 typedef enum { S_OK, S_SERVER_ERR, S_CLIENT_ERR } stream_status_e ;
 
 struct istream_private
@@ -53,11 +58,15 @@ struct intercept *si_init( serp )
 	ip->int_priv = (void *) &istream ;
 	ip->int_ops = &istream_ops ;
 	int_init( ip, serp ) ;
-	if ( (int) signal( SIGPIPE, SIG_IGN ) == -1 )
+	if ( signal( SIGPIPE, SIG_IGN ) == SIG_ERR )
 		int_fail( ip, "signal" ) ;
 	return( ip ) ;
 }
 
+PRIVATE status_e handle_io( psi_h iter, channel_s *chp, fd_set *maskp, stream_status_e (*iofunc)() );
+PRIVATE stream_status_e tcp_local_to_remote( channel_s *chp );
+PRIVATE stream_status_e tcp_remote_to_local( channel_s *chp );
+PRIVATE void connection_request( struct intercept *ip, channel_s **chpp );
 
 PRIVATE void si_exit()
 {
@@ -76,10 +85,6 @@ PRIVATE void si_mux()
 	int								mask_max ;
 	psi_h								iter ;
 	char								*func = "si_mux" ;
-	PRIVATE status_e				handle_io() ;
-	PRIVATE stream_status_e		tcp_local_to_remote() ;
-	PRIVATE stream_status_e		tcp_remote_to_local() ;
-	PRIVATE void					connection_request() ;
 
 	FD_ZERO( &socket_mask ) ;
 	FD_SET( INT_REMOTE( ip ), &socket_mask ) ;
@@ -129,8 +134,7 @@ PRIVATE void si_mux()
 					msg( LOG_DEBUG, func, "Input available on local socket %d", 
 																			chp->ch_local_socket ) ;
 #endif
-				if ( handle_io( iter, chp,
-								&socket_mask, tcp_local_to_remote ) == FAILED )
+				if ( handle_io( iter, chp, &socket_mask, tcp_local_to_remote ) == FAILED )
 					return ;
 				if ( --n_ready == 0 )
 					break ;
