@@ -4,18 +4,20 @@
  * and conditions for redistribution.
  */
 
-static char RCSid[] = "$Id: udpint.c,v 1.3 1995/09/10 18:41:13 chuck Exp $" ;
+static char RCSid[] = "$Id: udpint.c,v 1.1.1.1 1999/10/12 17:28:59 bbraun Exp $" ;
 
 #include <sys/types.h>
 #include <sys/socket.h>
-#if defined(BSD) || defined(linux)
 #include <netinet/in.h>
-#endif
 #include <netdb.h>
 #include <sys/time.h>
 #include <syslog.h>
 #include <errno.h>
 
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
+
 #include "access.h"
 #include "defs.h"
 #include "int.h"
@@ -52,6 +54,11 @@ static struct idgram_private idgram ;
 PRIVATE void di_mux() ;
 PRIVATE void di_exit() ;
 
+PRIVATE void udp_remote_to_local( struct intercept *ip, channel_s **chpp );
+PRIVATE status_e udp_local_to_remote( channel_s *chp );
+PRIVATE void send_data( int sd, char *buf, int len, struct sockaddr_in *addr );
+PRIVATE status_e get_incoming_packet( struct intercept *ip, packet_s *pp );
+
 static struct intercept_ops idgram_ops =
 	{
 		di_mux,
@@ -94,8 +101,6 @@ PRIVATE void di_mux()
 	register struct intercept	*ip = &dgram_intercept_state ;
 	fd_set							socket_mask ;
 	int								mask_max ;
-	PRIVATE void					udp_remote_to_local() ;
-	PRIVATE status_e				udp_local_to_remote() ;
 
 	FD_ZERO( &socket_mask ) ;
 	FD_SET( INT_REMOTE( ip ), &socket_mask ) ;
@@ -157,8 +162,6 @@ PRIVATE void udp_remote_to_local( ip, chpp )
 	packet_s				packet ;
 	channel_s			*chp ;
 	bool_int				addr_checked ;
-	PRIVATE void		send_data() ;
-	PRIVATE status_e	get_incoming_packet() ;
 
 	*chpp = CHANNEL_NULL ;
 
